﻿using System;
using System.ComponentModel;
using IdeaBlade.EntityModel;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Navigation;

namespace WindowsStoreDevTour
{
    public sealed partial class DetailPage : INotifyPropertyChanged
    {
        private Customer _customer;

        public DetailPage()
        {
            InitializeComponent();

            // Consider using an MVVM framework such as Cocktail instead of Code-Behind.
            DataContext = this;
        }

        public Customer Customer
        {
            get { return _customer; }
            set
            {
                if (Equals(value, _customer)) return;
                _customer = value;
                NotifyPropertyChange("Customer");
            }
        }

        public bool CanGoBack
        {
            get { return Frame != null && Frame.CanGoBack && !DataService.Instance.HasChanges(); }
        }

        public bool CanSave
        {
            get { return DataService.Instance.HasChanges(); }
        }

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);

            DataService.Instance.EntityChanged += OnEntityChanged;
            Start((Guid) e.Parameter);

            NotifyPropertyChange("CanGoBack");
        }

        protected override void OnNavigatedFrom(NavigationEventArgs e)
        {
            base.OnNavigatedFrom(e);

            DataService.Instance.EntityChanged -= OnEntityChanged;
        }

        private void OnEntityChanged(object sender, EntityChangedEventArgs entityChangedEventArgs)
        {
            NotifyPropertyChange("CanSave");
            NotifyPropertyChange("CanGoBack");
        }

        public async void Start(Guid customerId)
        {
            Customer = await DataService.Instance.GetCustomerAsync(customerId);
        }

        private void GoBack(object sender, RoutedEventArgs e)
        {
            Frame.GoBack();
        }

        private async void Save(object sender, RoutedEventArgs e)
        {
            await DataService.Instance.SaveAsync();
        }

        private void Discard(object sender, RoutedEventArgs e)
        {
            DataService.Instance.Discard();
        }

        private void NotifyPropertyChange(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null) handler(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}